@@ -14,6 +14,8 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+
+import monotonic
 from oslo_log import log
 
 import ceilometer
@@ -45,6 +47,7 @@ class CPUPollster(pollsters.BaseComputePollster):
                     unit='ns',
                     volume=cpu_info.time,
                     additional_metadata=cpu_num,
+                    monotonic_time=monotonic.monotonic()
                 )
             except virt_inspector.InstanceNotFoundException as err:
                 # Instance was deleted while getting samples. Ignore it.
@@ -105,7 +108,7 @@ class CPUL3CachePollster(pollsters.BaseComputePollster):
                 cpu_cache = self.inspector.inspect_cpu_l3_cache(instance)
                 LOG.debug(_("CPU cache size: %(id)s %(cache_size)d"),
                           ({'id': instance.id,
-                            'l3_cache_usage': cpu_cache.l3_cache_usage}))
+                            'cache_size': cpu_cache.l3_cache_usage}))
                 yield util.make_sample_from_instance(
                     instance,
                     name='cpu_l3_cache',
