@@ -17,10 +17,12 @@
 import abc
 import collections
 
+import monotonic
 from oslo_log import log
 import six
 
 import ceilometer
+from ceilometer.agent import plugin_base
 from ceilometer.compute import pollsters
 from ceilometer.compute.pollsters import util
 from ceilometer.compute.virt import inspector as virt_inspector
@@ -32,7 +34,7 @@ LOG = log.getLogger(__name__)
 
 DiskIOData = collections.namedtuple(
     'DiskIOData',
-    'r_bytes r_requests w_bytes w_requests per_disk_requests',
+    'r_bytes r_requests w_bytes w_requests per_disk_requests polled_time',
 )
 
 DiskRateData = collections.namedtuple('DiskRateData',
@@ -102,12 +104,14 @@ class _Base(pollsters.BaseComputePollster):
                 'write_bytes': per_device_write_bytes,
                 'write_requests': per_device_write_requests,
             }
+            polled_time = monotonic.monotonic()
             i_cache[instance.id] = DiskIOData(
                 r_bytes=r_bytes,
                 r_requests=r_requests,
                 w_bytes=w_bytes,
                 w_requests=w_requests,
                 per_disk_requests=per_device_requests,
+                polled_time=polled_time
             )
         return i_cache[instance.id]
 
@@ -127,23 +131,15 @@ class _Base(pollsters.BaseComputePollster):
             volume=getattr(c_data, _volume),
             additional_metadata={
                 'device': c_data.per_disk_requests[_metadata].keys()},
+            monotonic_time=c_data.polled_time,
         )]
 
-    @staticmethod
-    def _get_samples_per_device(c_data, _attr, instance, _name, _unit):
+    def _get_samples_per_device(self, c_data, _attr, instance, _name, _unit):
         """Return one or more Samples for meter 'disk.device.*'"""
-        samples = []
-        for disk, value in six.iteritems(c_data.per_disk_requests[_attr]):
-            samples.append(util.make_sample_from_instance(
-                instance,
-                name=_name,
-                type=sample.TYPE_CUMULATIVE,
-                unit=_unit,
-                volume=value,
-                resource_id="%s-%s" % (instance.id, disk),
-                additional_metadata={'disk_name': disk},
-            ))
-        return samples
+        return self._get_samples_per_devices(c_data.per_disk_requests[_attr],
+                                             instance, _name,
+                                             sample.TYPE_CUMULATIVE, _unit,
+                                             c_data.polled_time)
 
     def get_samples(self, manager, cache, resources):
         for instance in resources:
@@ -170,6 +166,7 @@ class _Base(pollsters.BaseComputePollster):
                           ' %(pollster)s',
                           {'inspector': self.inspector.__class__.__name__,
                            'pollster': self.__class__.__name__})
+                raise plugin_base.PollsterPermanentError(resources)
             except Exception as err:
                 LOG.exception(_('Ignoring instance %(name)s: %(error)s'),
                               {'name': instance_name, 'error': err})
@@ -307,6 +304,7 @@ class _DiskRatesPollsterBase(pollsters.BaseComputePollster):
                           ' %(pollster)s',
                           {'inspector': self.inspector.__class__.__name__,
                            'pollster': self.__class__.__name__})
+                raise plugin_base.PollsterPermanentError(resources)
             except Exception as err:
                 instance_name = util.instance_name(instance)
                 LOG.exception(_('Ignoring instance %(name)s: %(error)s'),
@@ -315,19 +313,9 @@ class _DiskRatesPollsterBase(pollsters.BaseComputePollster):
     def _get_samples_per_device(self, disk_rates_info, _attr, instance,
                                 _name, _unit):
         """Return one or more Samples for meter 'disk.device.*'."""
-        samples = []
-        for disk, value in six.iteritems(disk_rates_info.per_disk_rate[
-                _attr]):
-            samples.append(util.make_sample_from_instance(
-                instance,
-                name=_name,
-                type=sample.TYPE_GAUGE,
-                unit=_unit,
-                volume=value,
-                resource_id="%s-%s" % (instance.id, disk),
-                additional_metadata={'disk_name': disk},
-            ))
-        return samples
+        return self._get_samples_per_devices(
+            disk_rates_info.per_disk_rate[_attr],
+            instance, _name, sample.TYPE_GAUGE, _unit)
 
     def _get_sample_read_and_write(self, instance, _name, _unit, _element,
                                    _attr1, _attr2):
@@ -442,6 +430,7 @@ class _DiskLatencyPollsterBase(pollsters.BaseComputePollster):
                           ' %(pollster)s',
                           {'inspector': self.inspector.__class__.__name__,
                            'pollster': self.__class__.__name__})
+                raise plugin_base.PollsterPermanentError(resources)
             except Exception as err:
                 instance_name = util.instance_name(instance)
                 LOG.exception(_('Ignoring instance %(name)s: %(error)s'),
@@ -512,6 +501,7 @@ class _DiskIOPSPollsterBase(pollsters.BaseComputePollster):
                           '%(pollster)s',
                           {'inspector': self.inspector.__class__.__name__,
                            'pollster': self.__class__.__name__})
+                raise plugin_base.PollsterPermanentError(resources)
             except Exception as err:
                 instance_name = util.instance_name(instance)
                 LOG.exception(_('Ignoring instance %(name)s: %(error)s'),
@@ -640,6 +630,7 @@ class _DiskInfoPollsterBase(pollsters.BaseComputePollster):
                           ' %(pollster)s',
                           {'inspector': self.inspector.__class__.__name__,
                            'pollster': self.__class__.__name__})
+                raise plugin_base.PollsterPermanentError(resources)
             except Exception as err:
                 instance_name = util.instance_name(instance)
                 LOG.exception(_('Ignoring instance %(name)s '
