@@ -16,9 +16,11 @@
 
 import copy
 
+import monotonic
 from oslo_log import log
 
 import ceilometer
+from ceilometer.agent import plugin_base
 from ceilometer.compute import pollsters
 from ceilometer.compute.pollsters import util
 from ceilometer.compute.virt import inspector as virt_inspector
@@ -34,7 +36,8 @@ class _Base(pollsters.BaseComputePollster):
                                   "write-bytes=%d"])
 
     @staticmethod
-    def make_vnic_sample(instance, name, type, unit, volume, vnic_data):
+    def make_vnic_sample(instance, name, type, unit, volume, vnic_data,
+                         monotonic_time):
         metadata = copy.copy(vnic_data)
         additional_metadata = dict(zip(metadata._fields, metadata))
         if vnic_data.fref is not None:
@@ -52,7 +55,8 @@ class _Base(pollsters.BaseComputePollster):
             unit=unit,
             volume=volume,
             resource_id=rid,
-            additional_metadata=additional_metadata
+            additional_metadata=additional_metadata,
+            monotonic_time=monotonic_time,
         )
 
     CACHE_KEY_VNIC = 'vnics'
@@ -71,9 +75,10 @@ class _Base(pollsters.BaseComputePollster):
     def _get_vnics_for_instance(self, cache, inspector, instance):
         i_cache = cache.setdefault(self.CACHE_KEY_VNIC, {})
         if instance.id not in i_cache:
-            i_cache[instance.id] = list(
-                self._get_vnic_info(inspector, instance)
-            )
+            data = list(self._get_vnic_info(inspector, instance))
+            polled_time = monotonic.monotonic()
+            i_cache[instance.id] = [(vnic, info, polled_time)
+                                    for vnic, info in data]
         return i_cache[instance.id]
 
     def get_samples(self, manager, cache, resources):
@@ -87,11 +92,11 @@ class _Base(pollsters.BaseComputePollster):
                     self.inspector,
                     instance,
                 )
-                for vnic, info in vnics:
+                for vnic, info, polled_time in vnics:
                     LOG.debug(self.NET_USAGE_MESSAGE, instance_name,
                               vnic.name, self._get_rx_info(info),
                               self._get_tx_info(info))
-                    yield self._get_sample(instance, vnic, info)
+                    yield self._get_sample(instance, vnic, info, polled_time)
             except virt_inspector.InstanceNotFoundException as err:
                 # Instance was deleted while getting samples. Ignore it.
                 LOG.debug('Exception while getting samples %s', err)
@@ -106,6 +111,7 @@ class _Base(pollsters.BaseComputePollster):
                           ' %(pollster)s',
                           {'inspector': self.inspector.__class__.__name__,
                            'pollster': self.__class__.__name__})
+                raise plugin_base.PollsterPermanentError(resources)
             except Exception as err:
                 LOG.exception(_('Ignoring instance %(name)s: %(error)s'),
                               {'name': instance_name, 'error': err})
@@ -132,9 +138,24 @@ class _RateBase(_Base):
         return info.tx_bytes_rate
 
 
+class _PacketsBase(_Base):
+
+    NET_USAGE_MESSAGE = ' '.join(["NETWORK USAGE:", "%s %s:",
+                                  "read-packets=%d",
+                                  "write-packets=%d"])
+
+    @staticmethod
+    def _get_rx_info(info):
+        return info.rx_packets
+
+    @staticmethod
+    def _get_tx_info(info):
+        return info.tx_packets
+
+
 class IncomingBytesPollster(_Base):
 
-    def _get_sample(self, instance, vnic, info):
+    def _get_sample(self, instance, vnic, info, polled_time):
         return self.make_vnic_sample(
             instance,
             name='network.incoming.bytes',
@@ -142,12 +163,13 @@ class IncomingBytesPollster(_Base):
             unit='B',
             volume=info.rx_bytes,
             vnic_data=vnic,
+            monotonic_time=polled_time,
         )
 
 
-class IncomingPacketsPollster(_Base):
+class IncomingPacketsPollster(_PacketsBase):
 
-    def _get_sample(self, instance, vnic, info):
+    def _get_sample(self, instance, vnic, info, polled_time):
         return self.make_vnic_sample(
             instance,
             name='network.incoming.packets',
@@ -155,12 +177,13 @@ class IncomingPacketsPollster(_Base):
             unit='packet',
             volume=info.rx_packets,
             vnic_data=vnic,
+            monotonic_time=polled_time,
         )
 
 
 class OutgoingBytesPollster(_Base):
 
-    def _get_sample(self, instance, vnic, info):
+    def _get_sample(self, instance, vnic, info, polled_time):
         return self.make_vnic_sample(
             instance,
             name='network.outgoing.bytes',
@@ -168,12 +191,13 @@ class OutgoingBytesPollster(_Base):
             unit='B',
             volume=info.tx_bytes,
             vnic_data=vnic,
+            monotonic_time=polled_time,
         )
 
 
-class OutgoingPacketsPollster(_Base):
+class OutgoingPacketsPollster(_PacketsBase):
 
-    def _get_sample(self, instance, vnic, info):
+    def _get_sample(self, instance, vnic, info, polled_time):
         return self.make_vnic_sample(
             instance,
             name='network.outgoing.packets',
@@ -181,12 +205,13 @@ class OutgoingPacketsPollster(_Base):
             unit='packet',
             volume=info.tx_packets,
             vnic_data=vnic,
+            monotonic_time=polled_time,
         )
 
 
 class IncomingBytesRatePollster(_RateBase):
 
-    def _get_sample(self, instance, vnic, info):
+    def _get_sample(self, instance, vnic, info, polled_time):
         return self.make_vnic_sample(
             instance,
             name='network.incoming.bytes.rate',
@@ -194,12 +219,13 @@ class IncomingBytesRatePollster(_RateBase):
             unit='B/s',
             volume=info.rx_bytes_rate,
             vnic_data=vnic,
+            monotonic_time=polled_time,
         )
 
 
 class OutgoingBytesRatePollster(_RateBase):
 
-    def _get_sample(self, instance, vnic, info):
+    def _get_sample(self, instance, vnic, info, polled_time):
         return self.make_vnic_sample(
             instance,
             name='network.outgoing.bytes.rate',
@@ -207,4 +233,5 @@ class OutgoingBytesRatePollster(_RateBase):
             unit='B/s',
             volume=info.tx_bytes_rate,
             vnic_data=vnic,
+            monotonic_time=polled_time,
         )
