@@ -2,9 +2,6 @@
 # Copyright 2012 New Dream Network, LLC (DreamHost)
 # Copyright 2013 eNovance
 #
-# Authors: Doug Hellmann <doug.hellmann@dreamhost.com>
-#          Julien Danjou <julien@danjou.info>
-#
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
 # not use this file except in compliance with the License. You may obtain
 # a copy of the License at
@@ -27,7 +24,7 @@ import copy
 import uuid
 
 from oslo_config import cfg
-
+from oslo_utils import timeutils
 
 OPTS = [
     cfg.StrOpt('sample_source',
@@ -57,8 +54,8 @@ cfg.CONF.register_opts(OPTS)
 class Sample(object):
 
     def __init__(self, name, type, unit, volume, user_id, project_id,
-                 resource_id, timestamp, resource_metadata, source=None,
-                 id=None):
+                 resource_id, timestamp=None, resource_metadata=None,
+                 source=None, id=None, monotonic_time=None):
         self.name = name
         self.type = type
         self.unit = unit
@@ -67,9 +64,10 @@ class Sample(object):
         self.project_id = project_id
         self.resource_id = resource_id
         self.timestamp = timestamp
-        self.resource_metadata = resource_metadata
+        self.resource_metadata = resource_metadata or {}
         self.source = source or cfg.CONF.sample_source
         self.id = id or str(uuid.uuid1())
+        self.monotonic_time = monotonic_time
 
     def as_dict(self):
         return copy.copy(self.__dict__)
@@ -99,6 +97,13 @@ class Sample(object):
                    resource_metadata=metadata,
                    source=source)
 
+    def set_timestamp(self, timestamp):
+        self.timestamp = timestamp
+
+    def get_iso_timestamp(self):
+        return timeutils.parse_isotime(self.timestamp)
+
+
 TYPE_GAUGE = 'gauge'
 TYPE_DELTA = 'delta'
 TYPE_CUMULATIVE = 'cumulative'
