@@ -144,8 +144,12 @@ class Connection(hbase_base.Connection, base.Connection):
         """Write the data to the backend storage system.
 
         :param data: a dictionary such as returned by
-          ceilometer.meter.meter_message_from_counter
+          ceilometer.publisher.utils.meter_message_from_counter
         """
+
+        # We must not record thing.
+        data.pop("monotonic_time", None)
+
         with self.conn_pool.connection() as conn:
             resource_table = conn.table(self.RESOURCE_TABLE)
             meter_table = conn.table(self.METER_TABLE)
@@ -223,7 +227,7 @@ class Connection(hbase_base.Connection, base.Connection):
                 first_ts = min(meters, key=operator.itemgetter(1))[1]
                 last_ts = max(meters, key=operator.itemgetter(1))[1]
                 source = meters[0][0][1]
-                # If we use QualifierFilter then HBase returnes only
+                # If we use QualifierFilter then HBase returns only
                 # qualifiers filtered by. It will not return the whole entry.
                 # That's why if we need to ask additional qualifiers manually.
                 if 'project_id' not in f_res and 'user_id' not in f_res:
