@@ -26,7 +26,7 @@ class BinTestCase(base.BaseTestCase):
     def setUp(self):
         super(BinTestCase, self).setUp()
         content = ("[DEFAULT]\n"
-                   "rpc_backend=fake\n"
+                   "transport_url = fake://\n"
                    "[database]\n"
                    "connection=log://localhost\n")
         if six.PY3:
@@ -58,7 +58,7 @@ class BinTestCase(base.BaseTestCase):
 
     def _test_run_expirer_ttl_enabled(self, ttl_name, data_name):
         content = ("[DEFAULT]\n"
-                   "rpc_backend=fake\n"
+                   "transport_url = fake://\n"
                    "[database]\n"
                    "%s=1\n"
                    "connection=log://localhost\n" % ttl_name)
@@ -90,7 +90,7 @@ class BinSendSampleTestCase(base.BaseTestCase):
         super(BinSendSampleTestCase, self).setUp()
         pipeline_cfg_file = self.path_get('etc/ceilometer/pipeline.yaml')
         content = ("[DEFAULT]\n"
-                   "rpc_backend=fake\n"
+                   "transport_url = fake://\n"
                    "pipeline_cfg_file={0}\n".format(pipeline_cfg_file))
         if six.PY3:
             content = content.encode('utf-8')
@@ -128,7 +128,7 @@ class BinCeilometerPollingServiceTestCase(base.BaseTestCase):
 
     def test_starting_with_duplication_namespaces(self):
         content = ("[DEFAULT]\n"
-                   "rpc_backend=fake\n"
+                   "transport_url = fake://\n"
                    "[database]\n"
                    "connection=log://localhost\n")
         if six.PY3:
@@ -155,7 +155,7 @@ class BinCeilometerPollingServiceTestCase(base.BaseTestCase):
 
     def test_polling_namespaces_invalid_value_in_config(self):
         content = ("[DEFAULT]\n"
-                   "rpc_backend=fake\n"
+                   "transport_url = fake://\n"
                    "polling_namespaces = ['central']\n"
                    "[database]\n"
                    "connection=log://localhost\n")
