@@ -15,15 +15,12 @@
 """Tests for Ceilometer notify daemon."""
 
 import shutil
+import time
 
 import mock
 from oslo_config import fixture as fixture_config
-from oslo_context import context
 import oslo_messaging
-import oslo_messaging.conffixture
-import oslo_service.service
 from oslo_utils import fileutils
-from oslo_utils import timeutils
 import six
 from stevedore import extension
 import yaml
@@ -93,11 +90,12 @@ class TestNotification(tests_base.BaseTestCase):
         self.CONF = self.useFixture(fixture_config.Config()).conf
         self.CONF.set_override("connection", "log://", group='database')
         self.CONF.set_override("backend_url", None, group="coordination")
-        self.CONF.set_override("store_events", False, group="notification")
         self.CONF.set_override("disable_non_metric_meters", False,
                                group="notification")
+        self.CONF.set_override("workload_partitioning", True,
+                               group='notification')
         self.setup_messaging(self.CONF)
-        self.srv = notification.NotificationService()
+        self.srv = notification.NotificationService(0)
 
     def fake_get_notifications_manager(self, pm):
         self.plugin = instance.Instance(pm)
@@ -111,15 +109,16 @@ class TestNotification(tests_base.BaseTestCase):
         )
 
     @mock.patch('ceilometer.pipeline.setup_pipeline', mock.MagicMock())
-    @mock.patch.object(oslo_messaging.MessageHandlingServer, 'start',
-                       mock.MagicMock())
+    @mock.patch('ceilometer.pipeline.setup_event_pipeline', mock.MagicMock())
     @mock.patch('ceilometer.event.endpoint.EventsNotificationEndpoint')
     def _do_process_notification_manager_start(self,
                                                fake_event_endpoint_class):
+        self.CONF([], project='ceilometer', validate_default_values=True)
         with mock.patch.object(self.srv,
                                '_get_notifications_manager') as get_nm:
             get_nm.side_effect = self.fake_get_notifications_manager
-            self.srv.start()
+            self.srv.run()
+        self.addCleanup(self.srv.terminate)
         self.fake_event_endpoint = fake_event_endpoint_class.return_value
 
     def test_start_multiple_listeners(self):
@@ -138,24 +137,17 @@ class TestNotification(tests_base.BaseTestCase):
                            'payload': TEST_NOTICE_PAYLOAD,
                            'metadata': TEST_NOTICE_METADATA}])
 
-        self.assertEqual(1, len(self.srv.listeners[0].dispatcher.endpoints))
-        self.assertTrue(self.srv.pipeline_manager.publisher.called)
-
-    def test_process_notification_no_events(self):
-        self._do_process_notification_manager_start()
-        self.assertEqual(1, len(self.srv.listeners[0].dispatcher.endpoints))
-        self.assertNotEqual(self.fake_event_endpoint,
-                            self.srv.listeners[0].dispatcher.endpoints[0])
+        self.assertEqual(2, len(self.srv.listeners[0].dispatcher.endpoints))
 
     @mock.patch('ceilometer.pipeline.setup_event_pipeline', mock.MagicMock())
     def test_process_notification_with_events(self):
-        self.CONF.set_override("store_events", True, group="notification")
         self._do_process_notification_manager_start()
         self.assertEqual(2, len(self.srv.listeners[0].dispatcher.endpoints))
         self.assertEqual(self.fake_event_endpoint,
                          self.srv.listeners[0].dispatcher.endpoints[0])
 
     @mock.patch('ceilometer.pipeline.setup_pipeline', mock.MagicMock())
+    @mock.patch('ceilometer.pipeline.setup_event_pipeline', mock.MagicMock())
     @mock.patch('oslo_messaging.get_batch_notification_listener')
     def test_unique_consumers(self, mock_listener):
 
@@ -165,14 +157,15 @@ class TestNotification(tests_base.BaseTestCase):
                 [extension.Extension('test', None, None, plugin),
                  extension.Extension('test', None, None, plugin)])
 
+        self.CONF([], project='ceilometer', validate_default_values=True)
         with mock.patch.object(self.srv,
                                '_get_notifications_manager') as get_nm:
             get_nm.side_effect = fake_get_notifications_manager_dup_targets
-            self.srv.start()
-            self.assertEqual(1, len(mock_listener.call_args_list))
+            self.srv.run()
+            self.addCleanup(self.srv.terminate)
+            self.assertEqual(2, len(mock_listener.call_args_list))
             args, kwargs = mock_listener.call_args
-            self.assertEqual(1, len(args[1]))
-            self.assertIsInstance(args[1][0], oslo_messaging.Target)
+            self.assertEqual(1, len(self.srv.listeners))
 
 
 class BaseRealNotification(tests_base.BaseTestCase):
@@ -230,7 +223,6 @@ class BaseRealNotification(tests_base.BaseTestCase):
         self.expected_samples = 2
 
         self.CONF.set_override("backend_url", None, group="coordination")
-        self.CONF.set_override("store_events", True, group="notification")
         self.CONF.set_override("disable_non_metric_meters", False,
                                group="notification")
 
@@ -247,18 +239,18 @@ class BaseRealNotification(tests_base.BaseTestCase):
         self.publisher = test_publisher.TestPublisher("")
 
     def _check_notification_service(self):
-        self.srv.start()
+        self.srv.run()
+        self.addCleanup(self.srv.terminate)
 
         notifier = messaging.get_notifier(self.transport,
                                           "compute.vagrant-precise")
-        notifier.info(context.RequestContext(), 'compute.instance.create.end',
+        notifier.info({}, 'compute.instance.create.end',
                       TEST_NOTICE_PAYLOAD)
-        start = timeutils.utcnow()
-        while timeutils.delta_seconds(start, timeutils.utcnow()) < 600:
+        start = time.time()
+        while time.time() - start < 60:
             if (len(self.publisher.samples) >= self.expected_samples and
                     len(self.publisher.events) >= self.expected_events):
                 break
-        self.srv.stop()
 
         resources = list(set(s.resource_id for s in self.publisher.samples))
         self.assertEqual(self.expected_samples, len(self.publisher.samples))
@@ -273,67 +265,67 @@ class TestRealNotificationReloadablePipeline(BaseRealNotification):
         self.CONF.set_override('refresh_pipeline_cfg', True)
         self.CONF.set_override('refresh_event_pipeline_cfg', True)
         self.CONF.set_override('pipeline_polling_interval', 1)
-        self.srv = notification.NotificationService()
+        self.srv = notification.NotificationService(0)
 
     @mock.patch('ceilometer.publisher.test.TestPublisher')
     def test_notification_pipeline_poller(self, fake_publisher_cls):
         fake_publisher_cls.return_value = self.publisher
-        self.srv.tg = mock.MagicMock()
-        self.srv.start()
-
-        pipeline_poller_call = mock.call(1, self.srv.refresh_pipeline)
-        self.assertIn(pipeline_poller_call,
-                      self.srv.tg.add_timer.call_args_list)
-        self.srv.stop()
+        self.srv.run()
+        self.addCleanup(self.srv.terminate)
+        self.assertIsNotNone(self.srv.refresh_pipeline_periodic)
 
     def test_notification_reloaded_pipeline(self):
         pipeline_cfg_file = self.setup_pipeline(['instance'])
         self.CONF.set_override("pipeline_cfg_file", pipeline_cfg_file)
 
-        self.srv.start()
+        self.srv.run()
+        self.addCleanup(self.srv.terminate)
 
-        pipeline = self.srv.pipe_manager
+        pipeline = self.srv.pipeline_manager.cfg_hash
 
         # Modify the collection targets
         updated_pipeline_cfg_file = self.setup_pipeline(['vcpus',
                                                          'disk.root.size'])
-        # Move/re-name the updated pipeline file to the original pipeline
+        # Move/rename the updated pipeline file to the original pipeline
         # file path as recorded in oslo config
         shutil.move(updated_pipeline_cfg_file, pipeline_cfg_file)
-        self.srv.refresh_pipeline()
-
-        self.assertNotEqual(pipeline, self.srv.pipe_manager)
-        self.srv.stop()
+        start = time.time()
+        while time.time() - start < 10:
+            if pipeline != self.srv.pipeline_manager.cfg_hash:
+                break
+        else:
+            self.fail("Pipeline failed to reload")
 
     def test_notification_reloaded_event_pipeline(self):
         ev_pipeline_cfg_file = self.setup_event_pipeline(
             ['compute.instance.create.start'])
         self.CONF.set_override("event_pipeline_cfg_file", ev_pipeline_cfg_file)
 
-        self.CONF.set_override("store_events", True, group="notification")
+        self.srv.run()
+        self.addCleanup(self.srv.terminate)
 
-        self.srv.start()
-
-        pipeline = self.srv.event_pipe_manager
+        pipeline = self.srv.event_pipeline_manager.cfg_hash
 
         # Modify the collection targets
         updated_ev_pipeline_cfg_file = self.setup_event_pipeline(
             ['compute.instance.*'])
 
-        # Move/re-name the updated pipeline file to the original pipeline
+        # Move/rename the updated pipeline file to the original pipeline
         # file path as recorded in oslo config
         shutil.move(updated_ev_pipeline_cfg_file, ev_pipeline_cfg_file)
-        self.srv.refresh_pipeline()
-
-        self.assertNotEqual(pipeline, self.srv.pipe_manager)
-        self.srv.stop()
+        start = time.time()
+        while time.time() - start < 10:
+            if pipeline != self.srv.event_pipeline_manager.cfg_hash:
+                break
+        else:
+            self.fail("Pipeline failed to reload")
 
 
 class TestRealNotification(BaseRealNotification):
 
     def setUp(self):
         super(TestRealNotification, self).setUp()
-        self.srv = notification.NotificationService()
+        self.srv = notification.NotificationService(0)
 
     @mock.patch('ceilometer.publisher.test.TestPublisher')
     def test_notification_service(self, fake_publisher_cls):
@@ -343,16 +335,16 @@ class TestRealNotification(BaseRealNotification):
     @mock.patch('ceilometer.publisher.test.TestPublisher')
     def test_notification_service_error_topic(self, fake_publisher_cls):
         fake_publisher_cls.return_value = self.publisher
-        self.srv.start()
+        self.srv.run()
+        self.addCleanup(self.srv.terminate)
         notifier = messaging.get_notifier(self.transport,
                                           'compute.vagrant-precise')
-        notifier.error(context.RequestContext(), 'compute.instance.error',
+        notifier.error({}, 'compute.instance.error',
                        TEST_NOTICE_PAYLOAD)
-        start = timeutils.utcnow()
-        while timeutils.delta_seconds(start, timeutils.utcnow()) < 600:
+        start = time.time()
+        while time.time() - start < 60:
             if len(self.publisher.events) >= self.expected_events:
                 break
-        self.srv.stop()
         self.assertEqual(self.expected_events, len(self.publisher.events))
 
     @mock.patch('ceilometer.publisher.test.TestPublisher')
@@ -365,14 +357,6 @@ class TestRealNotification(BaseRealNotification):
         self._check_notification_service()
         self.assertEqual('memory', self.publisher.samples[0].name)
 
-    @mock.patch.object(oslo_service.service.Service, 'stop')
-    def test_notification_service_start_abnormal(self, mocked):
-        try:
-            self.srv.stop()
-        except Exception:
-            pass
-        self.assertEqual(1, mocked.call_count)
-
 
 class TestRealNotificationHA(BaseRealNotification):
 
@@ -380,13 +364,23 @@ class TestRealNotificationHA(BaseRealNotification):
         super(TestRealNotificationHA, self).setUp()
         self.CONF.set_override('workload_partitioning', True,
                                group='notification')
-        self.srv = notification.NotificationService()
+        self.srv = notification.NotificationService(0)
 
     @mock.patch('ceilometer.publisher.test.TestPublisher')
     def test_notification_service(self, fake_publisher_cls):
         fake_publisher_cls.return_value = self.publisher
         self._check_notification_service()
 
+    @mock.patch.object(oslo_messaging.MessageHandlingServer, 'start')
+    def test_notification_threads(self, m_listener):
+        self.CONF.set_override('batch_size', 1, group='notification')
+        self.srv.run()
+        m_listener.assert_called_with(override_pool_size=None)
+        m_listener.reset_mock()
+        self.CONF.set_override('batch_size', 2, group='notification')
+        self.srv.run()
+        m_listener.assert_called_with(override_pool_size=1)
+
     @mock.patch('oslo_messaging.get_batch_notification_listener')
     def test_reset_listener_on_refresh(self, mock_listener):
         mock_listener.side_effect = [
@@ -395,7 +389,8 @@ class TestRealNotificationHA(BaseRealNotification):
             mock.MagicMock(),  # refresh pipeline listener
         ]
 
-        self.srv.start()
+        self.srv.run()
+        self.addCleanup(self.srv.terminate)
 
         def _check_listener_targets():
             args, kwargs = mock_listener.call_args
@@ -407,15 +402,15 @@ class TestRealNotificationHA(BaseRealNotification):
         listener = self.srv.pipeline_listener
         self.srv._configure_pipeline_listener()
         self.assertIsNot(listener, self.srv.pipeline_listener)
-        _check_listener_targets()
 
-        self.srv.stop()
+        _check_listener_targets()
 
     @mock.patch('oslo_messaging.get_batch_notification_listener')
     def test_retain_common_targets_on_refresh(self, mock_listener):
         with mock.patch('ceilometer.coordination.PartitionCoordinator'
                         '.extract_my_subset', return_value=[1, 2]):
-            self.srv.start()
+            self.srv.run()
+            self.addCleanup(self.srv.terminate)
         listened_before = [target.topic for target in
                            mock_listener.call_args[0][1]]
         self.assertEqual(4, len(listened_before))
@@ -428,11 +423,11 @@ class TestRealNotificationHA(BaseRealNotification):
         common = set(listened_before) & set(listened_after)
         for topic in common:
             self.assertTrue(topic.endswith('1'))
-        self.srv.stop()
 
     @mock.patch('oslo_messaging.get_batch_notification_listener')
     def test_notify_to_relevant_endpoint(self, mock_listener):
-        self.srv.start()
+        self.srv.run()
+        self.addCleanup(self.srv.terminate)
 
         targets = mock_listener.call_args[0][1]
         self.assertIsNotEmpty(targets)
@@ -452,11 +447,10 @@ class TestRealNotificationHA(BaseRealNotification):
                                                      notifier.publisher_id,
                                                      None, None, None))
 
-        self.srv.stop()
-
     @mock.patch('oslo_messaging.Notifier.sample')
     def test_broadcast_to_relevant_pipes_only(self, mock_notifier):
-        self.srv.start()
+        self.srv.run()
+        self.addCleanup(self.srv.terminate)
         for endpoint in self.srv.listeners[0].dispatcher.endpoints:
             if (hasattr(endpoint, 'filter_rule') and
                 not endpoint.filter_rule.match(None, None, 'nonmatching.end',
@@ -490,7 +484,6 @@ class TestRealNotificationHA(BaseRealNotification):
                          mock_notifier.call_args_list[1][1]['event_type'])
         self.assertEqual('ceilometer.pipeline',
                          mock_notifier.call_args_list[2][1]['event_type'])
-        self.srv.stop()
 
 
 class TestRealNotificationMultipleAgents(tests_base.BaseTestCase):
@@ -516,6 +509,18 @@ class TestRealNotificationMultipleAgents(tests_base.BaseTestCase):
                                                         suffix="yaml")
         return pipeline_cfg_file
 
+    def setup_event_pipeline(self):
+        pipeline = yaml.dump({
+            'sources': [],
+            'sinks': []
+        })
+        if six.PY3:
+            pipeline = pipeline.encode('utf-8')
+
+        pipeline_cfg_file = fileutils.write_to_tempfile(
+            content=pipeline, prefix="event_pipeline", suffix="yaml")
+        return pipeline_cfg_file
+
     def setUp(self):
         super(TestRealNotificationMultipleAgents, self).setUp()
         self.CONF = self.useFixture(fixture_config.Config()).conf
@@ -523,9 +528,11 @@ class TestRealNotificationMultipleAgents(tests_base.BaseTestCase):
         self.setup_messaging(self.CONF, 'nova')
 
         pipeline_cfg_file = self.setup_pipeline(['instance', 'memory'])
+        event_pipeline_cfg_file = self.setup_event_pipeline()
         self.CONF.set_override("pipeline_cfg_file", pipeline_cfg_file)
+        self.CONF.set_override("event_pipeline_cfg_file",
+                               event_pipeline_cfg_file)
         self.CONF.set_override("backend_url", None, group="coordination")
-        self.CONF.set_override("store_events", False, group="notification")
         self.CONF.set_override("disable_non_metric_meters", False,
                                group="notification")
         self.CONF.set_override('workload_partitioning', True,
@@ -538,34 +545,32 @@ class TestRealNotificationMultipleAgents(tests_base.BaseTestCase):
     def _check_notifications(self, fake_publisher_cls):
         fake_publisher_cls.side_effect = [self.publisher, self.publisher2]
 
-        self.srv = notification.NotificationService()
-        self.srv2 = notification.NotificationService()
+        self.srv = notification.NotificationService(0)
+        self.srv2 = notification.NotificationService(0)
         with mock.patch('ceilometer.coordination.PartitionCoordinator'
                         '._get_members', return_value=['harry', 'lloyd']):
             with mock.patch('uuid.uuid4', return_value='harry'):
-                self.srv.start()
+                self.srv.run()
+            self.addCleanup(self.srv.terminate)
             with mock.patch('uuid.uuid4', return_value='lloyd'):
-                self.srv2.start()
+                self.srv2.run()
+            self.addCleanup(self.srv2.terminate)
 
         notifier = messaging.get_notifier(self.transport,
                                           "compute.vagrant-precise")
         payload1 = TEST_NOTICE_PAYLOAD.copy()
         payload1['instance_id'] = '0'
-        notifier.info(context.RequestContext(), 'compute.instance.create.end',
-                      payload1)
+        notifier.info({}, 'compute.instance.create.end', payload1)
         payload2 = TEST_NOTICE_PAYLOAD.copy()
         payload2['instance_id'] = '1'
-        notifier.info(context.RequestContext(), 'compute.instance.create.end',
-                      payload2)
+        notifier.info({}, 'compute.instance.create.end', payload2)
         self.expected_samples = 4
-        start = timeutils.utcnow()
         with mock.patch('six.moves.builtins.hash', lambda x: int(x)):
-            while timeutils.delta_seconds(start, timeutils.utcnow()) < 60:
+            start = time.time()
+            while time.time() - start < 60:
                 if (len(self.publisher.samples + self.publisher2.samples) >=
                         self.expected_samples):
                     break
-            self.srv.stop()
-            self.srv2.stop()
 
         self.assertEqual(2, len(self.publisher.samples))
         self.assertEqual(2, len(self.publisher2.samples))
